@@ -4,7 +4,7 @@
 //  About box handling for Aven.
 //
 //  Copyright (C) 2001-2003 Mark R. Shinwell.
-//  Copyright (C) 2001,2002,2003,2004,2005,2006,2010,2014 Olly Betts
+//  Copyright (C) 2001,2002,2003,2004,2005,2006,2010,2014,2015 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -37,14 +37,12 @@
 BEGIN_EVENT_TABLE(AboutDlg, wxDialog)
     EVT_TIMER(about_TIMER, AboutDlg::OnTimer)
     EVT_BUTTON(wxID_COPY, AboutDlg::OnCopy)
-    EVT_BUTTON(wxID_CLOSE, AboutDlg::OnClose)
 END_EVENT_TABLE()
 
 void
 AboutDlg::OnTimer(wxTimerEvent &)
 {
-    wxImage::AddHandler(new wxJPEGHandler);
-    bitmap.LoadFile(icon_path + wxT("osterei.jpg"), wxBITMAP_TYPE_JPEG);
+    bitmap.LoadFile(img_path + wxT("osterei.png"), wxBITMAP_TYPE_PNG);
     ((wxStaticBitmap*)FindWindowById(501, this))->SetBitmap(bitmap);
 }
 
@@ -59,23 +57,21 @@ AboutDlg::OnCopy(wxCommandEvent &)
     }
 }
 
-void
-AboutDlg::OnClose(wxCommandEvent &)
-{
-    Destroy();
-}
-
-AboutDlg::AboutDlg(wxWindow* parent, const wxString & icon_path_) :
+AboutDlg::AboutDlg(wxWindow* parent, const wxIcon & app_icon) :
     /* TRANSLATORS: for the title of the About box */
     wxDialog(parent, 500, wxString::Format(wmsg(/*About %s*/205), APP_NAME)),
-    icon_path(icon_path_), timer(this, about_TIMER)
+    timer(this, about_TIMER)
 {
+    img_path = wxString(wmsg_cfgpth());
+    img_path += wxCONFIG_PATH_SEPARATOR;
+    img_path += wxT("images");
+    img_path += wxCONFIG_PATH_SEPARATOR;
+
     wxBoxSizer* horiz = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* vert = new wxBoxSizer(wxVERTICAL);
 
     if (!bitmap.Ok()) {
-	bitmap.LoadFile(icon_path + APP_ABOUT_IMAGE, wxBITMAP_TYPE_PNG);
-	bitmap_icon.LoadFile(icon_path + APP_IMAGE, wxBITMAP_TYPE_PNG);
+	bitmap.LoadFile(img_path + APP_ABOUT_IMAGE, wxBITMAP_TYPE_PNG);
     }
     if (bitmap.Ok()) {
 	wxStaticBitmap* static_bitmap = new wxStaticBitmap(this, 501, bitmap);
@@ -84,7 +80,7 @@ AboutDlg::AboutDlg(wxWindow* parent, const wxString & icon_path_) :
     }
     horiz->Add(vert, 0, wxALL, 2);
 
-    wxString id(APP_NAME wxT(" "VERSION"\n"));
+    wxString id(APP_NAME wxT(" " VERSION "\n"));
     /* TRANSLATORS: Here "survey" is a "cave map" rather than list of questions
      * - it should be translated to the terminology that cavers using the
      * language would use.
@@ -93,15 +89,18 @@ AboutDlg::AboutDlg(wxWindow* parent, const wxString & icon_path_) :
      */
     id += wmsg(/*Survey visualisation tool*/209);
     wxBoxSizer* title = new wxBoxSizer(wxHORIZONTAL);
-    if (bitmap_icon.Ok()) {
-	title->Add(new wxStaticBitmap(this, 599, bitmap_icon), 0, wxALIGN_CENTRE_VERTICAL|wxRIGHT, 8);
-    }
+    wxStaticBitmap* static_bitmap = new wxStaticBitmap(this, 599, wxBitmap());
+    static_bitmap->SetIcon(app_icon);
+    title->Add(static_bitmap, 0, wxALIGN_CENTRE_VERTICAL|wxRIGHT, 8);
     title->Add(new wxStaticText(this, 502, id), 0, wxALL, 2);
 
     wxStaticText* copyright = new wxStaticText(this, 503,
-					wxT(COPYRIGHT_MSG_UTF8"\n"AVEN_COPYRIGHT_MSG_UTF8));
+					wxT(COPYRIGHT_MSG_UTF8 "\n" AVEN_COPYRIGHT_MSG_UTF8));
 
     wxString licence_str;
+    /* TRANSLATORS: Summary paragraph for the GPLv2 - there are translations for
+     * some languages here:
+     * http://www.gnu.org/licenses/old-licenses/gpl-2.0-translations.html */
     wxString l(wmsg(/*This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public Licence as published by the Free Software Foundation; either version 2 of the Licence, or (at your option) any later version.*/219));
     wxClientDC dc(this);
     dc.SetFont(this->GetFont());
@@ -195,10 +194,12 @@ AboutDlg::AboutDlg(wxWindow* parent, const wxString & icon_path_) :
     wxBoxSizer* bottom = new wxBoxSizer(wxHORIZONTAL);
     bottom->Add(5, 5, 1);
     bottom->Add(new wxButton(this, wxID_COPY), 0, wxRIGHT | wxBOTTOM, 6);
-    wxButton* close = new wxButton(this, wxID_CLOSE);
+    wxButton* close = new wxButton(this, wxID_OK);
     bottom->Add(close, 0, wxRIGHT | wxBOTTOM, 15);
     vert->Add(bottom, 0, wxEXPAND | wxLEFT | wxRIGHT, 0);
-    vert->SetMinSize(0, bitmap.GetHeight());
+    if (bitmap.Ok()) {
+	vert->SetMinSize(0, bitmap.GetHeight());
+    }
 
     SetSizer(horiz);
     close->SetDefault();
