@@ -1,6 +1,6 @@
 /* commands.c
  * Code for directives
- * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013,2014 Olly Betts
+ * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -256,7 +256,7 @@ typedef enum {
    CMD_TEAM, CMD_TITLE, CMD_TRUNCATE, CMD_UNITS
 } cmds;
 
-static sztok cmd_tab[] = {
+static const sztok cmd_tab[] = {
      {"ALIAS",     CMD_ALIAS},
      {"BEGIN",     CMD_BEGIN},
      {"CALIBRATE", CMD_CALIBRATE},
@@ -296,15 +296,42 @@ static sztok cmd_tab[] = {
 #define ANG_UMASK (BIT(UNITS_DEGS) | BIT(UNITS_GRADS) | BIT(UNITS_MINUTES))
 
 /* ordering must be the same as the units enum */
-static real factor_tab[] = {
+const real factor_tab[] = {
    1.0, METRES_PER_FOOT, (METRES_PER_FOOT*3.0),
    (M_PI/180.0), (M_PI/200.0), 0.01, (M_PI/180.0/60.0)
 };
 
+const int units_to_msgno[] = {
+    /*m*/424,
+    /*ft*/428,
+    -1, /* yards */
+    /*°*/344,
+    /*ᵍ*/76,
+    /*%*/96,
+    -1 /* minutes */
+};
+
+int get_length_units(int quantity) {
+    double factor = pcs->units[quantity];
+    if (fabs(factor - METRES_PER_FOOT) <= REAL_EPSILON ||
+	fabs(factor - METRES_PER_FOOT * 3.0) <= REAL_EPSILON) {
+	return UNITS_FEET;
+    }
+    return UNITS_METRES;
+}
+
+int get_angle_units(int quantity) {
+    double factor = pcs->units[quantity];
+    if (fabs(factor - M_PI / 200.0) <= REAL_EPSILON) {
+	return UNITS_GRADS;
+    }
+    return UNITS_DEGS;
+}
+
 static int
 get_units(unsigned long qmask, bool percent_ok)
 {
-   static sztok utab[] = {
+   static const sztok utab[] = {
 	{"DEGREES",       UNITS_DEGS },
 	{"DEGS",	  UNITS_DEGS },
 	{"FEET",	  UNITS_FEET },
@@ -347,7 +374,7 @@ get_units(unsigned long qmask, bool percent_ok)
 static unsigned long
 get_qlist(unsigned long mask_bad)
 {
-   static sztok qtab[] = {
+   static const sztok qtab[] = {
 	{"ALTITUDE",	 Q_DZ },
 	{"BACKBEARING",  Q_BACKBEARING },
 	{"BACKCLINO",    Q_BACKGRADIENT },    /* alternative name */
@@ -407,6 +434,8 @@ get_qlist(unsigned long mask_bad)
 
    if (qmask == 0) {
       file.lpos += strlen(buffer);
+      /* TRANSLATORS: A "quantity" is something measured like "LENGTH",
+       * "BEARING", "ALTITUDE", etc. */
       compile_error_skip(-/*Unknown quantity “%s”*/34, buffer);
    } else {
       set_pos(&fp);
@@ -419,7 +448,7 @@ get_qlist(unsigned long mask_bad)
 static void
 cmd_set(void)
 {
-   static sztok chartab[] = {
+   static const sztok chartab[] = {
 	{"BLANK",     SPECIAL_BLANK },
 /*FIXME	{"CLOSE",     SPECIAL_CLOSE }, */
 	{"COMMENT",   SPECIAL_COMMENT },
@@ -523,21 +552,50 @@ check_reentry(prefix *survey)
       if (reenter_depr_count >= 5)
 	 return;
 
-      /* TRANSLATORS: e.g.
+      /* TRANSLATORS: The first of two warnings given when a survey which has
+       * already been completed is reentered.  This example file crawl.svx:
        *
        * *begin crawl
-       * 1 2 9.45 234 -01
+       * 1 2 9.45 234 -01 # <- second warning here
        * *end crawl
-       * *begin crawl    # <- warning here
+       * *begin crawl     # <- first warning here
        * 2 3 7.67 223 -03
        * *end crawl
-       * 
+       *
+       * Would lead to:
+       *
+       * crawl.svx:3: Reentering an existing survey is deprecated
+       * crawl.svx:1: Originally entered here
+       *
        * If you're unsure what "deprecated" means, see:
        * http://en.wikipedia.org/wiki/Deprecation */
       compile_warning(/*Reentering an existing survey is deprecated*/29);
+      /* TRANSLATORS: The second of two warnings given when a survey which has
+       * already been completed is reentered.  This example file crawl.svx:
+       *
+       * *begin crawl
+       * 1 2 9.45 234 -01 # <- second warning here
+       * *end crawl
+       * *begin crawl     # <- first warning here
+       * 2 3 7.67 223 -03
+       * *end crawl
+       *
+       * Would lead to:
+       *
+       * crawl.svx:3: Reentering an existing survey is deprecated
+       * crawl.svx:1: Originally entered here
+       *
+       * If you're unsure what "deprecated" means, see:
+       * http://en.wikipedia.org/wiki/Deprecation */
       compile_warning_pfx(survey, /*Originally entered here*/30);
-      if (++reenter_depr_count == 5)
+      if (++reenter_depr_count == 5) {
+	 /* After we've warned about 5 uses of the same deprecated feature, we
+	  * give up for the rest of the current processing run.
+	  *
+	  * If you're unsure what "deprecated" means, see:
+	  * http://en.wikipedia.org/wiki/Deprecation */
 	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
+      }
    } else {
       survey->sflags |= BIT(SFLAGS_PREFIX_ENTERED);
       survey->filename = file.filename;
@@ -760,6 +818,10 @@ cmd_fix(void)
       y = read_numeric(fFalse, &ny);
       z = read_numeric(fFalse, &nz);
       sdx = read_numeric(fTrue, NULL);
+      if (sdx <= 0) {
+	  compile_error_skip(-/*Standard deviation must be positive*/48);
+	  return;
+      }
       if (sdx != HUGE_REAL) {
 	 real sdy, sdz;
 	 real cxy = 0, cyz = 0, czx = 0;
@@ -768,12 +830,20 @@ cmd_fix(void)
 	    /* only one variance given */
 	    sdy = sdz = sdx;
 	 } else {
+	    if (sdy <= 0) {
+	       compile_error_skip(-/*Standard deviation must be positive*/48);
+	       return;
+	    }
 	    sdz = read_numeric(fTrue, NULL);
 	    if (sdz == HUGE_REAL) {
 	       /* two variances given - horizontal & vertical */
 	       sdz = sdy;
 	       sdy = sdx;
 	    } else {
+	       if (sdz <= 0) {
+		  compile_error_skip(-/*Standard deviation must be positive*/48);
+		  return;
+	       }
 	       cxy = read_numeric(fTrue, NULL);
 	       if (cxy != HUGE_REAL) {
 		  /* covariances given */
@@ -861,7 +931,7 @@ cmd_fix(void)
 static void
 cmd_flags(void)
 {
-   static sztok flagtab[] = {
+   static const sztok flagtab[] = {
 	{"DUPLICATE", FLAGS_DUPLICATE },
 	{"NOT",	      FLAGS_NOT },
 	{"SPLAY",     FLAGS_SPLAY },
@@ -917,7 +987,10 @@ cmd_equate(void)
       if (name1 == NULL) {
 	 if (fOnlyOneStn) {
 	    /* TRANSLATORS: EQUATE is a command name, so shouldn’t be
-	     * translated. */
+	     * translated.
+	     *
+	     * Here "station" is a survey station, not a train station.
+	     */
 	    compile_error_skip(-/*Only one station in EQUATE command*/33);
 	 }
 	 return;
@@ -952,7 +1025,11 @@ report_missing_export(prefix *pfx, int depth)
        * *export 1
        * 1 2 1.23 045 -6
        * *end inner
-       * *end outer */
+       * *end outer
+       *
+       * Here "survey" is a "cave map" rather than list of questions - it should be
+       * translated to the terminology that cavers using the language would use.
+       */
       compile_error_pfx(survey,
 			/*Station “%s” not exported from survey “%s”*/26, p, s);
    } else {
@@ -1017,7 +1094,7 @@ cmd_export(void)
 static void
 cmd_data(void)
 {
-   static sztok dtab[] = {
+   static const sztok dtab[] = {
 	{"ALTITUDE",	 Dz },
 	{"BACKBEARING",  BackComp },
 	{"BACKCLINO",    BackClino }, /* alternative name */
@@ -1064,11 +1141,11 @@ cmd_data(void)
 #define MASK_clin BIT(Clino) | BIT(BackClino)
 
 #define MASK_NORMAL MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_clin
-#define MASK_DIVING MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_dpth
+#define MASK_DIVING MASK_NORMAL | MASK_dpth
 #define MASK_CARTESIAN MASK_stns | BIT(Dx) | BIT(Dy) | BIT(Dz)
 #define MASK_CYLPOLAR  MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_dpth
-#define MASK_PASSAGE BIT(Station) | BIT(Left) | BIT(Right) | BIT(Up) | BIT(Down)
 #define MASK_NOSURVEY MASK_stns
+#define MASK_PASSAGE BIT(Station) | BIT(Left) | BIT(Right) | BIT(Up) | BIT(Down)
 
    /* readings which may be given for each style */
    static const unsigned long mask[] = {
@@ -1079,7 +1156,7 @@ cmd_data(void)
    /* readings which may be omitted for each style */
    static const unsigned long mask_optional[] = {
       BIT(Dir) | BIT(Clino) | BIT(BackClino),
-      BIT(Dir),
+      BIT(Dir) | BIT(Clino) | BIT(BackClino),
       0,
       BIT(Dir),
       0,
@@ -1098,7 +1175,7 @@ cmd_data(void)
 #define STYLE_DEFAULT   -2
 #define STYLE_UNKNOWN   -1
 
-   static sztok styletab[] = {
+   static const sztok styletab[] = {
 	{"CARTESIAN",    STYLE_CARTESIAN },
 	{"CYLPOLAR",     STYLE_CYLPOLAR },
 	{"DEFAULT",      STYLE_DEFAULT },
@@ -1263,6 +1340,11 @@ cmd_data(void)
 	    if (fBad) {
 	       /* Not entirely happy with phrasing this... */
 	       file.lpos += strlen(buffer);
+	       /* TRANSLATORS: This is an error from the *DATA command.  It
+		* means that a reading (which will appear where %s is isn't
+		* valid as the list of readings has already included the same
+		* reading, or an equivalent one (e.g. you can't have both
+		* DEPTH and DEPTHCHANGE together). */
 	       compile_error_skip(-/*Reading “%s” duplicates previous reading(s)*/77,
 			     buffer);
 	       osfree(style_name);
@@ -1410,7 +1492,6 @@ cmd_units(void)
       compile_error_skip(-/**UNITS factor must be non-zero*/200);
       return;
    }
-   if (factor == HUGE_REAL) factor = (real)1.0;
 
    units = get_units(qmask, fTrue);
    if (units == UNITS_NULL) return;
@@ -1419,7 +1500,11 @@ cmd_units(void)
    if (TSTBIT(qmask, Q_BACKGRADIENT))
       pcs->f_backclino_percent = (units == UNITS_PERCENT);
 
-   factor *= factor_tab[units];
+   if (factor == HUGE_REAL) {
+      factor = factor_tab[units];
+   } else {
+      factor *= factor_tab[units];
+   }
 
    for (quantity = 0, m = BIT(quantity); m <= qmask; quantity++, m <<= 1)
       if (qmask & m) pcs->units[quantity] = factor;
@@ -1441,6 +1526,10 @@ cmd_calibrate(void)
    }
 
    if (((qmask & LEN_QMASK)) && ((qmask & ANG_QMASK))) {
+      /* TRANSLATORS: e.g.
+       *
+       * *calibrate tape compass 1 1
+       */
       compile_error_skip(/*Can’t calibrate angular and length quantities together*/227);
       return;
    }
@@ -1474,7 +1563,7 @@ cmd_calibrate(void)
 static void
 cmd_default(void)
 {
-   static sztok defaulttab[] = {
+   static const sztok defaulttab[] = {
       { "CALIBRATE", CMD_CALIBRATE },
       { "DATA",      CMD_DATA },
       { "UNITS",     CMD_UNITS },
@@ -1589,7 +1678,7 @@ cmd_title(void)
    }
 }
 
-static sztok case_tab[] = {
+static const sztok case_tab[] = {
      {"PRESERVE", OFF},
      {"TOLOWER",  LOWER},
      {"TOUPPER",  UPPER},
@@ -1627,7 +1716,7 @@ typedef enum {
     CS_UTM
 } cs_class;
 
-static sztok cs_tab[] = {
+static const sztok cs_tab[] = {
      {"CUSTOM", CS_CUSTOM},
      {"EPSG",   CS_EPSG},	/* EPSG:<number> */
      {"ESRI",   CS_ESRI},	/* ESRI:<number> */
@@ -1909,7 +1998,7 @@ cmd_cs(void)
    }
 }
 
-static sztok infer_tab[] = {
+static const sztok infer_tab[] = {
      { "EQUATES",	INFER_EQUATES },
      { "EXPORTS",	INFER_EXPORTS },
      { "PLUMBS",	INFER_PLUMBS },
@@ -1919,7 +2008,7 @@ static sztok infer_tab[] = {
      { NULL,		INFER_NULL }
 };
 
-static sztok onoff_tab[] = {
+static const sztok onoff_tab[] = {
      { "OFF", 0 },
      { "ON",  1 },
      { NULL, -1 }
@@ -2002,7 +2091,12 @@ cmd_require(void)
 	  * greater" if that gives a more natural translation.  It's
 	  * technically not quite right when there are parallel active release
 	  * series (e.g. Survex 1.0.40 was released *after* 1.2.0), but this
-	  * seems unlikely to confuse users. */
+	  * seems unlikely to confuse users.  "Survex" is the name of the
+	  * software, so should not be translated.
+	  *
+	  * Here "survey" is a "cave map" rather than list of questions - it should be
+	  * translated to the terminology that cavers using the language would use.
+	  */
 	 fatalerror_in_file(file.filename, file.line, /*Survex version %s or greater required to process this survey data.*/2, v);
       }
       if (ch != '.') break;
@@ -2075,7 +2169,7 @@ read:
 
 typedef void (*cmd_fn)(void);
 
-static cmd_fn cmd_funcs[] = {
+static const cmd_fn cmd_funcs[] = {
    cmd_alias,
    cmd_begin,
    cmd_calibrate,
