@@ -1,6 +1,6 @@
 /* datain.c
  * Reads in survey files, dealing with special characters, keywords & data
- * Copyright (C) 1991-2003,2005,2009,2010,2011,2012,2013,2014 Olly Betts
+ * Copyright (C) 1991-2003,2005,2009,2010,2011,2012,2013,2014,2015 Olly Betts
  * Copyright (C) 2004 Simeon Warner
  *
  * This program is free software; you can redistribute it and/or modify
@@ -103,7 +103,9 @@ report_parent(parse * p) {
      * parent files */
     p->reported_where = fFalse;
     /* TRANSLATORS: %s is replaced by the filename of the parent file, and %u
-     * by the line number in that file. */
+     * by the line number in that file.  Your translation should also contain
+     * %s:%u so that automatic parsing of error messages to determine the file
+     * and line number still works. */
     fprintf(STDERR, msg(/*In file included from %s:%u:\n*/5), p->filename, p->line);
 }
 
@@ -127,7 +129,7 @@ compile_error(int en, ...)
    error_list_parent_files();
    if (en < 0) {
       en = -en;
-      col = ftell(file.fh) - file.lpos;
+      if (file.fh) col = ftell(file.fh) - file.lpos;
    }
    v_report(1, file.filename, file.line, col, en, ap);
    va_end(ap);
@@ -142,7 +144,7 @@ compile_error_skip(int en, ...)
    error_list_parent_files();
    if (en < 0) {
       en = -en;
-      col = ftell(file.fh) - file.lpos;
+      if (file.fh) col = ftell(file.fh) - file.lpos;
    }
    v_report(1, file.filename, file.line, col, en, ap);
    va_end(ap);
@@ -190,7 +192,7 @@ compile_warning(int en, ...)
    error_list_parent_files();
    if (en < 0) {
       en = -en;
-      col = ftell(file.fh) - file.lpos;
+      if (file.fh) col = ftell(file.fh) - file.lpos;
    }
    v_report(0, file.filename, file.line, col, en, ap);
    va_end(ap);
@@ -396,7 +398,7 @@ data_file(const char *pth, const char *fnm)
       }
 
       if (fh == NULL) {
-	 compile_error(/*Couldn’t open file “%s”*/24, fnm);
+	 compile_error(-/*Couldn’t open file “%s”*/24, fnm);
 	 return;
       }
 
@@ -617,6 +619,7 @@ data_file(const char *pth, const char *fnm)
 	    }
 	    data_normal();
 	 }
+	 clear_last_leg();
       }
       {
 	 settings *pcsParent = pcs->next;
@@ -738,6 +741,7 @@ data_file(const char *pth, const char *fnm)
 	    }
 	 }
       }
+      clear_last_leg();
    }
 
    /* don't allow *BEGIN at the end of a file, then *EXPORT in the
@@ -834,19 +838,23 @@ handle_plumb(clino_type *p_ctype)
 }
 
 static void
-warn_readings_differ(int msgno, real diff)
+warn_readings_differ(int msgno, real diff, int units)
 {
    char buf[64];
    char *p;
-   sprintf(buf, "%.2f", deg(fabs(diff)));
-   p = strchr(buf, '.');
-   if (p) {
-      char *z = p;
-      while (*++p) {
-	 if (*p != '0') z = p + 1;
+   diff /= get_units_factor(units);
+   sprintf(buf, "%.2f", fabs(diff));
+   for (p = buf; *p; ++p) {
+      if (*p == '.') {
+	 char *z = p;
+	 while (*++p) {
+	    if (*p != '0') z = p + 1;
+	 }
+	 p = z;
+	 break;
       }
-      *z = '\0';
    }
+   strcpy(p, get_units_string(units));
    compile_warning(msgno, buf);
 }
 
@@ -895,11 +903,12 @@ handle_compass(real *p_var)
 	 real diff = comp - backcomp;
 	 real adj = fabs(diff) > M_PI ? M_PI : 0;
 	 diff -= floor((diff + M_PI) / (2 * M_PI)) * 2 * M_PI;
-	 if (sqrd(diff / 2.0) > compvar + VAR(Q_BACKBEARING)) {
-	    /* fore and back readings differ by more than 2 sds */
-	    /* TRANSLATORS: Degrees are the angular measurement where there are
-	     * 360 in a full circle. */
-	    warn_readings_differ(/*COMPASS reading and BACKCOMPASS reading disagree by %s degrees*/98, diff);
+	 if (sqrd(diff / 3.0) > compvar + VAR(Q_BACKBEARING)) {
+	    /* fore and back readings differ by more than 3 sds */
+	    /* TRANSLATORS: %s is replaced by the amount the readings disagree
+	     * by, e.g. "2.5°" or "3ᵍ". */
+	    warn_readings_differ(/*COMPASS reading and BACKCOMPASS reading disagree by %s*/98,
+				 diff, get_angle_units(Q_BEARING));
 	 }
 	 comp = (comp / compvar + backcomp / VAR(BackComp));
 	 compvar = (compvar + VAR(BackComp)) / 4;
@@ -959,18 +968,30 @@ process_normal(prefix *fr, prefix *to, bool fToFirst,
       range_0_180 = (z > M_PI_4 && z < 3*M_PI_4);
       diff_from_abs90 = fabs(clin) - M_PI_2;
       if (diff_from_abs90 > EPSILON) {
-	 if (!range_0_180)
-	    /* TRANSLATORS: Degrees are the angular measurement where there are
-	     * 360 in a full circle. */
-	    compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
+	 if (!range_0_180) {
+	    int clino_units = get_angle_units(Q_GRADIENT);
+	    const char * units = get_units_string(clino_units);
+	    real right_angle = M_PI_2 / get_units_factor(clino_units);
+	    /* TRANSLATORS: %.f%s will be replaced with a right angle in the
+	     * units currently in use, e.g. "90°" or "100ᵍ".  And "absolute
+	     * value" means the reading ignoring the sign (so it might be
+	     * < -90° or > 90°. */
+	    compile_warning(/*Clino reading over %.f%s (absolute value)*/51,
+			    right_angle, units);
+	 }
       } else if (TSTBIT(pcs->infer, INFER_PLUMBS) &&
 		 diff_from_abs90 >= -EPSILON) {
 	 ctype = CTYPE_INFERPLUMB;
       }
       if (range_0_180 && ctype != CTYPE_INFERPLUMB) {
 	 /* FIXME: Warning message not ideal... */
-	 if (clin < 0.0 || clin - M_PI > EPSILON)
-	    compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
+	 if (clin < 0.0 || clin - M_PI > EPSILON) {
+	    int clino_units = get_angle_units(Q_GRADIENT);
+	    const char * units = get_units_string(clino_units);
+	    real right_angle = M_PI_2 / get_units_factor(clino_units);
+	    compile_warning(/*Clino reading over %.f%s (absolute value)*/51,
+			    right_angle, units);
+	 }
       }
    }
 
@@ -978,11 +999,16 @@ process_normal(prefix *fr, prefix *to, bool fToFirst,
       backclin *= pcs->units[Q_BACKGRADIENT];
       /* percentage scale */
       if (pcs->f_backclino_percent) backclin = atan(backclin);
+      /* FIXME: Add range_0_180 handling here too */
       if (ctype != CTYPE_READING) {
 	 real diff_from_abs90 = fabs(backclin) - M_PI_2;
 	 if (diff_from_abs90 > EPSILON) {
 	    /* FIXME: different message for BackClino? */
-	    compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
+	    int clino_units = get_angle_units(Q_BACKGRADIENT);
+	    const char * units = get_units_string(clino_units);
+	    real right_angle = M_PI_2 / get_units_factor(clino_units);
+	    compile_warning(/*Clino reading over %.f%s (absolute value)*/51,
+			    right_angle, units);
 	 } else if (TSTBIT(pcs->infer, INFER_PLUMBS) &&
 		    diff_from_abs90 >= -EPSILON) {
 	    backctype = CTYPE_INFERPLUMB;
@@ -1073,9 +1099,10 @@ process_normal(prefix *fr, prefix *to, bool fToFirst,
 	    if (ctype == CTYPE_READING) {
 	       if (sqrd((clin + backclin) / 3.0) > var_clin + VAR(BackClino)) {
 		  /* fore and back readings differ by more than 3 sds */
-		  /* TRANSLATORS: Degrees are the angular measurement where
-		   * there are 360 in a full circle. */
-		  warn_readings_differ(/*CLINO reading and BACKCLINO reading disagree by %s degrees*/99, clin + backclin);
+		  /* TRANSLATORS: %s is replaced by the amount the readings disagree
+		   * by, e.g. "2.5°" or "3ᵍ". */
+		  warn_readings_differ(/*CLINO reading and BACKCLINO reading disagree by %s*/99,
+				       clin + backclin, get_angle_units(Q_GRADIENT));
 	       }
 	       clin = (clin / var_clin - backclin / VAR(BackClino));
 	       var_clin = (var_clin + VAR(BackClino)) / 4;
@@ -1587,12 +1614,12 @@ data_normal(void)
 		nextch();
 		while (ch >= 'A' && ch <= 'Z') {
 		   compass_dat_flags |= BIT(ch - 'A');
-		   /* FIXME: we currently understand:
+		   /* We currently understand:
 		    *   L (exclude from length)
 		    *   X (exclude data)
-		    * but should also handle at least some of:
-		    *   C (no adjustment)
-		    *   P (no plot)
+		    * FIXME: but should also handle at least some of:
+		    *   C (no adjustment) (set all (co)variances to 0?)
+		    *   P (no plot) (new flag in 3d for "hidden by default"?)
 		    */
 		   nextch();
 		}
@@ -1654,6 +1681,7 @@ data_normal(void)
 				   ctype, backctype);
 		break;
 	      case STYLE_DIVING:
+		/* FIXME: Handle any clino readings */
 		r = process_diving(fr, to, (first_stn == To) ^ fRev,
 				   fDepthChange);
 		break;
@@ -1748,6 +1776,7 @@ data_normal(void)
 			       ctype, backctype);
 		break;
 	      case STYLE_DIVING:
+		/* FIXME: Handle any clino readings */
 		process_diving(fr, to, (first_stn == To) ^ fRev,
 			       fDepthChange);
 		break;
