@@ -2,7 +2,7 @@
  * Export to CAD-like formats (DXF, Skencil, SVG, EPS) and also Compass PLT.
  */
 
-/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011,2012,2013,2014 Olly Betts
+/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011,2012,2013,2014,2015 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,8 +33,10 @@
 #include "exportfilter.h"
 #include "gpx.h"
 #include "hpgl.h"
+#include "json.h"
 #include "kml.h"
 #include "mainfrm.h"
+#include "pos.h"
 
 #include <float.h>
 #include <math.h>
@@ -49,6 +51,9 @@
 # include <unistd.h>
 #endif
 
+#include <utility>
+#include <vector>
+
 #include "cmdline.h"
 #include "debug.h"
 #include "filename.h"
@@ -103,18 +108,8 @@ static const char *layer_name(int mask) {
     return "";
 }
 
-/* bounds */
-static double min_x, min_y, min_z, max_x, max_y, max_z;
-
-static double text_height; /* for station labels */
 static double marker_size; /* for station markers */
 static double grid; /* grid spacing (or 0 for no grid) */
-static double scale = 500.0;
-static double factor;
-static const char *unit = "mm";
-const double SVG_MARGIN = 5.0; // In units of "unit".
-
-static const char *survey = NULL;
 
 const int *
 ExportFilter::passes() const
@@ -125,13 +120,18 @@ ExportFilter::passes() const
 
 class DXF : public ExportFilter {
     const char * to_close;
+    /* for station labels */
+    double text_height;
     char pending[1024];
 
   public:
-    DXF() : to_close(0) { pending[0] = '\0'; }
+    explicit DXF(double text_height_)
+	: to_close(0), text_height(text_height_) { pending[0] = '\0'; }
     const int * passes() const;
     bool fopen(const char *fnm_out);
-    void header(const char *, const char *, time_t);
+    void header(const char *, const char *, time_t,
+		double min_x, double min_y, double min_z,
+		double max_x, double max_y, double max_z);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
@@ -160,7 +160,9 @@ DXF::fopen(const char *fnm_out)
 }
 
 void
-DXF::header(const char *, const char *, time_t)
+DXF::header(const char *, const char *, time_t,
+	    double min_x, double min_y, double min_z,
+	    double max_x, double max_y, double max_z)
 {
    fprintf(fh, "0\nSECTION\n"
 	       "2\nHEADER\n");
@@ -391,10 +393,14 @@ DXF::footer()
 }
 
 class Skencil : public ExportFilter {
+    double factor;
   public:
-    Skencil() { }
+    explicit Skencil(double scale)
+	: factor(POINTS_PER_MM * 1000.0 / scale) { }
     const int * passes() const;
-    void header(const char *, const char *, time_t);
+    void header(const char *, const char *, time_t,
+		double min_x, double min_y, double min_z,
+		double max_x, double max_y, double max_z);
     void start_pass(int layer);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
@@ -410,7 +416,9 @@ Skencil::passes() const
 }
 
 void
-Skencil::header(const char *, const char *, time_t)
+Skencil::header(const char *, const char *, time_t,
+		double min_x, double min_y, double /*min_z*/,
+		double max_x, double max_y, double /*max_z*/)
 {
    fprintf(fh, "##Sketch 1 2\n"); /* File format version */
    fprintf(fh, "document()\n");
@@ -544,12 +552,23 @@ find_name(const img_point *p)
 class SVG : public ExportFilter {
     const char * to_close;
     bool close_g;
+    double factor;
+    /* for station labels */
+    double text_height;
     char pending[1024];
 
   public:
-    SVG() : to_close(NULL), close_g(false) { pending[0] = '\0'; }
+    SVG(double scale, double text_height_)
+	: to_close(NULL),
+	  close_g(false),
+	  factor(1000.0 / scale),
+	  text_height(text_height_) {
+	pending[0] = '\0';
+    }
     const int * passes() const;
-    void header(const char *, const char *, time_t);
+    void header(const char *, const char *, time_t,
+		double min_x, double min_y, double min_z,
+		double max_x, double max_y, double max_z);
     void start_pass(int layer);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
@@ -571,11 +590,14 @@ SVG::passes() const
 }
 
 void
-SVG::header(const char * title, const char *, time_t)
+SVG::header(const char * title, const char *, time_t,
+	    double min_x, double min_y, double /*min_z*/,
+	    double max_x, double max_y, double /*max_z*/)
 {
-   size_t i;
+   const char *unit = "mm";
+   const double SVG_MARGIN = 5.0; // In units of "unit".
    htab = (point **)osmalloc(HTAB_SIZE * ossizeof(point *));
-   for (i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
+   for (size_t i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
    fprintf(fh, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
    double width = (max_x - min_x) * factor + SVG_MARGIN * 2;
    double height = (max_y - min_y) * factor + SVG_MARGIN * 2;
@@ -732,10 +754,14 @@ class PLT : public ExportFilter {
 
     const char * find_name_plt(const img_point *p);
 
+    double min_N, max_N, min_E, max_E, min_A, max_A;
+
   public:
     PLT() { }
     const int * passes() const;
-    void header(const char *, const char *, time_t);
+    void header(const char *, const char *, time_t,
+		double min_x, double min_y, double min_z,
+		double max_x, double max_y, double max_z);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
     void footer();
@@ -749,16 +775,23 @@ PLT::passes() const
 }
 
 void
-PLT::header(const char *title, const char *, time_t)
+PLT::header(const char *title, const char *, time_t,
+	    double min_x, double min_y, double min_z,
+	    double max_x, double max_y, double max_z)
 {
-   size_t i;
+   // FIXME: allow survey to be set from aven somehow!
+   const char *survey = NULL;
    htab = (point **)osmalloc(HTAB_SIZE * ossizeof(point *));
-   for (i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
+   for (size_t i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
    /* Survex is E, N, Alt - PLT file is N, E, Alt */
+   min_N = min_y / METRES_PER_FOOT;
+   max_N = max_y / METRES_PER_FOOT;
+   min_E = min_x / METRES_PER_FOOT;
+   max_E = max_x / METRES_PER_FOOT;
+   min_A = min_z / METRES_PER_FOOT;
+   max_A = max_z / METRES_PER_FOOT;
    fprintf(fh, "Z %.3f %.3f %.3f %.3f %.3f %.3f\r\n",
-	   min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
-	   min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
-	   min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
+	   min_N, max_N, min_E, max_E, min_A, max_A);
    fprintf(fh, "N%s D 1 1 1 C%s\r\n", survey ? survey : "X",
 	   (title && title[0]) ? title : "X");
 }
@@ -820,30 +853,51 @@ PLT::footer(void)
 {
    /* Survex is E, N, Alt - PLT file is N, E, Alt */
    fprintf(fh, "X %.3f %.3f %.3f %.3f %.3f %.3f\r\n",
-	   min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
-	   min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
-	   min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
+	   min_N, max_N, min_E, max_E, min_A, max_A);
    /* Yucky DOS "end of textfile" marker */
    PUTC('\x1a', fh);
 }
 
 class EPS : public ExportFilter {
+    double factor;
+    bool first;
+    vector<pair<double, double> > psg;
   public:
-    EPS() { }
-    void header(const char *, const char *, time_t);
+    explicit EPS(double scale)
+	: factor(POINTS_PER_MM * 1000.0 / scale) { }
+    const int * passes() const;
+    void header(const char *, const char *, time_t,
+		double min_x, double min_y, double min_z,
+		double max_x, double max_y, double max_z);
+    void start_pass(int layer);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
+    void xsect(const img_point *, double, double, double);
+    void wall(const img_point *, double, double);
+    void passage(const img_point *, double, double, double);
+    void tube_end();
     void footer();
 };
 
+const int *
+EPS::passes() const
+{
+    static const int eps_passes[] = {
+	PASG, XSECT, WALL1, WALL2, LEGS|SURF|STNS|LABELS, 0
+    };
+    return eps_passes;
+}
+
 void
-EPS::header(const char *title, const char *, time_t)
+EPS::header(const char *title, const char *, time_t,
+	    double min_x, double min_y, double /*min_z*/,
+	    double max_x, double max_y, double /*max_z*/)
 {
    const char * fontname_labels = "helvetica"; // FIXME
    int fontsize_labels = 10; // FIXME
    fputs("%!PS-Adobe-2.0 EPSF-1.2\n", fh);
-   fputs("%%Creator: Survex "VERSION" EPS Output Filter\n", fh);
+   fputs("%%Creator: Survex " VERSION " EPS Export Filter\n", fh);
 
    if (title && title[0])
        fprintf(fh, "%%%%Title: %s\n", title);
@@ -1005,8 +1059,6 @@ EPS::header(const char *title, const char *, time_t)
 
    fprintf(fh, "/lab findfont %d scalefont setfont\n", int(fontsize_labels));
 
-   fprintf(fh, "0.1 setlinewidth\n");
-
 #if 0
    /* C<digit> changes colour */
    /* FIXME: read from ini */
@@ -1031,8 +1083,10 @@ EPS::header(const char *title, const char *, time_t)
 
    /* define some functions to keep file short */
    fputs("/M {stroke moveto} def\n"
+	 "/P {stroke newpath moveto} def\n"
+	 "/F {closepath gsave 0.8 setgray fill grestore} def\n"
 	 "/L {lineto} def\n"
-/*	 "/R {rlineto} def\n" */
+	 "/R {rlineto} def\n"
 	 "/S {show} def\n", fh);
 
    fprintf(fh, "gsave %.8f dup scale\n", factor);
@@ -1072,6 +1126,20 @@ EPS::header(const char *title, const char *, time_t)
 #endif
 }
 
+void
+EPS::start_pass(int layer)
+{
+    first = true;
+    switch (layer) {
+	case LEGS|SURF|STNS|LABELS:
+	    fprintf(fh, "0.1 setlinewidth\n");
+	    break;
+	case PASG: case XSECT: case WALL1: case WALL2:
+	    fprintf(fh, "0.01 setlinewidth\n");
+	    break;
+    }
+}
+
 void
 EPS::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
@@ -1109,6 +1177,52 @@ EPS::cross(const img_point *p, bool fSurface)
    fprintf(fh, "%.2f %.2f X\n", p->x, p->y);
 }
 
+void
+EPS::xsect(const img_point *p, double angle, double d1, double d2)
+{
+    double s = sin(rad(angle));
+    double c = cos(rad(angle));
+    fprintf(fh, "%.2f %.2f M %.2f %.2f R\n",
+	    p->x - c * d2, p->y - s * d2,
+	    c * (d1 + d2), s * (d1 + d2));
+}
+
+void
+EPS::wall(const img_point *p, double angle, double d)
+{
+    double s = sin(rad(angle));
+    double c = cos(rad(angle));
+    fprintf(fh, "%.2f %.2f %c\n", p->x + c * d, p->y + s * d, first ? 'M' : 'L');
+    first = false;
+}
+
+void
+EPS::passage(const img_point *p, double angle, double d1, double d2)
+{
+    double s = sin(rad(angle));
+    double c = cos(rad(angle));
+    double x1 = p->x + c * d1;
+    double y1 = p->y + s * d1;
+    double x2 = p->x - c * d2;
+    double y2 = p->y - s * d2;
+    fprintf(fh, "%.2f %.2f %c\n", x1, y1, first ? 'P' : 'L');
+    first = false;
+    psg.push_back(make_pair(x2, y2));
+}
+
+void
+EPS::tube_end()
+{
+    if (!psg.empty()) {
+	vector<pair<double, double> >::const_reverse_iterator i;
+	for (i = psg.rbegin(); i != psg.rend(); ++i) {
+	    fprintf(fh, "%.2f %.2f L\n", i->first, i->second);
+	}
+	fputs("F\n", fh);
+	psg.clear();
+    }
+}
+
 void
 EPS::footer(void)
 {
@@ -1124,7 +1238,8 @@ Export(const wxString &fnm_out, const wxString &title,
        const MainFrm * mainfrm,
        double pan, double tilt, int show_mask, export_format format,
        const char * input_projection,
-       double grid_, double text_height_, double marker_size_)
+       double grid_, double text_height, double marker_size_,
+       double scale)
 {
    int fPendingMove = 0;
    img_point p, p1;
@@ -1133,26 +1248,15 @@ Export(const wxString &fnm_out, const wxString &title,
    bool elevation = (tilt == 0.0);
 
    grid = grid_;
-   text_height = text_height_;
    marker_size = marker_size_;
 
-#if 0 // FIXME: allow these to be set from aven somehow!
-       case 'r': /* Reduction factor */
-	 scale = cmdline_double_arg();
-	 break;
-       case 's':
-	 survey = optarg;
-	 break;
-#endif
-
    ExportFilter * filt;
    switch (format) {
        case FMT_DXF:
-	   filt = new DXF;
+	   filt = new DXF(text_height);
 	   break;
        case FMT_EPS:
-	   filt = new EPS;
-	   factor = POINTS_PER_MM * 1000.0 / scale;
+	   filt = new EPS(scale);
 	   break;
        case FMT_GPX:
 	   filt = new GPX(input_projection);
@@ -1162,6 +1266,9 @@ Export(const wxString &fnm_out, const wxString &title,
 	   filt = new HPGL;
 	   // factor = POINTS_PER_MM * 1000.0 / scale;
 	   break;
+       case FMT_JSON:
+	   filt = new JSON;
+	   break;
        case FMT_KML:
 	   filt = new KML(input_projection);
 	   show_mask |= FULL_COORDS;
@@ -1170,13 +1277,15 @@ Export(const wxString &fnm_out, const wxString &title,
 	   filt = new PLT;
 	   show_mask |= FULL_COORDS;
 	   break;
+       case FMT_POS:
+	   filt = new POS(mainfrm->GetSeparator());
+	   show_mask |= FULL_COORDS;
+	   break;
        case FMT_SK:
-	   filt = new Skencil;
-	   factor = POINTS_PER_MM * 1000.0 / scale;
+	   filt = new Skencil(scale);
 	   break;
        case FMT_SVG:
-	   filt = new SVG;
-	   factor = 1000.0 / scale;
+	   filt = new SVG(scale, text_height);
 	   break;
        default:
 	   return false;
@@ -1194,7 +1303,8 @@ Export(const wxString &fnm_out, const wxString &title,
       c = cos(rad(pan));
    }
 
-   /* Get drawing corners */
+   /* Get bounding box */
+   double min_x, min_y, min_z, max_x, max_y, max_z;
    min_x = min_y = min_z = HUGE_VAL;
    max_x = max_y = max_z = -HUGE_VAL;
    list<traverse>::const_iterator trav = mainfrm->traverses_begin();
@@ -1286,7 +1396,8 @@ Export(const wxString &fnm_out, const wxString &title,
    max_z += z_offset;
 
    /* Header */
-   filt->header(title.mb_str(), datestamp.mb_str(), datestamp_numeric);
+   filt->header(title.mb_str(), datestamp.mb_str(), datestamp_numeric,
+		min_x, min_y, min_z, max_x, max_y, max_z);
 
    p1.x = p1.y = p1.z = 0; /* avoid compiler warning */
 
