@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001,2002,2005 Mark R. Shinwell.
-//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013,2014 Olly Betts
+//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013,2014,2015 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -91,6 +91,8 @@ enum {
     COLOUR_BY_DEPTH,
     COLOUR_BY_DATE,
     COLOUR_BY_ERROR,
+    COLOUR_BY_GRADIENT,
+    COLOUR_BY_LENGTH,
     COLOUR_BY_LIMIT_ // Leave this last.
 };
 
@@ -106,6 +108,11 @@ enum {
       SPLAYS_SHOW_NORMAL,
 };
 
+// It's pointless to redraw the screen as often as we can on a fast machine,
+// since the display hardware will only update so many times per second.
+// This is the maximum framerate we'll redraw at.
+const int MAX_FRAMERATE = 50;
+
 class GfxCore : public GLACanvas {
     Double m_Scale;
     int m_ScaleBarWidth;
@@ -118,6 +125,8 @@ class GfxCore : public GLACanvas {
 	LIST_DEPTH_KEY,
 	LIST_DATE_KEY,
 	LIST_ERROR_KEY,
+	LIST_GRADIENT_KEY,
+	LIST_LENGTH_KEY,
 	LIST_UNDERGROUND_LEGS,
 	LIST_TUBES,
 	LIST_SURFACE_LEGS,
@@ -125,11 +134,25 @@ class GfxCore : public GLACanvas {
 	LIST_CROSSES,
 	LIST_GRID,
 	LIST_SHADOW,
+	LIST_TERRAIN,
 	LIST_LIMIT_ // Leave this last.
     } drawing_list;
 
     static const int NUM_COLOUR_BANDS = 13;
 
+    void SetPanBase() {
+	base_pan = m_PanAngle;
+	base_pan_time = timer.Time() - (1000 / MAX_FRAMERATE);
+    }
+
+    void SetTiltBase() {
+	base_tilt = m_TiltAngle;
+	base_tilt_time = timer.Time() - (1000 / MAX_FRAMERATE);
+    }
+
+    int GetCompassWidth() const;
+    int GetClinoWidth() const;
+
 public:
     typedef enum {
 	CURSOR_DEFAULT,
@@ -166,6 +189,7 @@ private:
     int m_ColourBy;
 
     bool m_HaveData;
+    bool m_HaveTerrain;
     bool m_MouseOutsideCompass;
     bool m_MouseOutsideElev;
     bool m_Surface;
@@ -174,12 +198,14 @@ private:
     bool m_ExportedPts;
     bool m_Grid;
     bool m_BoundingBox;
+    bool m_Terrain;
 
     bool m_Degrees;
     bool m_Metric;
     bool m_Percent;
 
     bool m_HitTestDebug;
+    bool m_RenderStats;
 
     list<LabelInfo*> *m_PointGrid;
     bool m_HitTestGridValid;
@@ -189,7 +215,10 @@ private:
     const LabelInfo * m_there;
 
     wxStopWatch timer;
-    long drawtime;
+    long base_tilt_time;
+    long base_pan_time;
+    Double base_tilt;
+    Double base_pan;
 
     GLAPen m_Pens[NUM_COLOUR_BANDS + 1];
 
@@ -214,6 +243,15 @@ private:
 
     ZoomBox zoombox;
 
+    // DEM:
+    unsigned short * dem;
+    unsigned long dem_width, dem_height;
+    double o_x, o_y, step_x, step_y;
+    long nodata_value;
+    bool bigendian;
+    long last_time;
+    size_t n_tris;
+
     void PlaceVertexWithColour(const Vector3 &v, Double factor = 1.0);
     void PlaceVertexWithColour(const Vector3 & v, GLint tex_x, GLint tex_y,
 			       Double factor);
@@ -221,8 +259,12 @@ private:
     void PlaceVertexWithDepthColour(const Vector3 & v, Double factor = 1.0);
     void PlaceVertexWithDepthColour(const Vector3 & v, GLint tex_x, GLint tex_y, Double factor);
 
+    void SetColourFrom01(double how_far, Double factor);
+
     void SetColourFromDate(int date, Double factor);
     void SetColourFromError(double E, Double factor);
+    void SetColourFromGradient(double angle, Double factor);
+    void SetColourFromLength(double len, Double factor);
 
     int GetClinoOffset() const;
     void DrawTick(int angle_cw);
@@ -234,6 +276,8 @@ private:
     void GenerateDisplayList();
     void GenerateDisplayListTubes();
     void GenerateDisplayListSurface();
+    void DrawTerrainTriangle(const Vector3 & a, const Vector3 & b, const Vector3 & c);
+    void DrawTerrain();
     void GenerateDisplayListShadow();
     void GenerateBlobsDisplayList();
 
@@ -247,6 +291,8 @@ private:
     void DrawDepthKey();
     void DrawDateKey();
     void DrawErrorKey();
+    void DrawGradientKey();
+    void DrawLengthKey();
     void DrawCompass();
     void DrawClino();
     void DrawClinoBack();
@@ -286,7 +332,6 @@ public:
     ~GfxCore();
 
     void Initialise(bool same_file);
-    void InitialiseTerrain();
 
     void UpdateBlobs();
     void ForceRefresh();
@@ -323,7 +368,7 @@ public:
     void OnMouseWheel(wxMouseEvent& event) { SetFocus(); m_Control->OnMouseWheel(event); }
     void OnKeyPress(wxKeyEvent &event) { m_Control->OnKeyPress(event); }
 
-    bool Animate();
+    void Animate();
     bool Animating() const {
 	return m_Rotating || m_SwitchingTo || presentation_mode != 0;
     }
@@ -371,6 +416,7 @@ public:
 
     bool IsRotating() const { return m_Rotating; }
     bool HasData() const { return m_DoneFirstShow && m_HaveData; }
+    bool HasTerrain() const { return m_DoneFirstShow && m_HaveTerrain; }
     bool HasDepth() const;
     bool HasErrorInformation() const;
     bool HasDateInformation() const;
@@ -437,18 +483,27 @@ public:
     void ToggleMetric() {
 	ToggleFlag(&m_Metric);
 	InvalidateList(LIST_DEPTH_KEY);
+	InvalidateList(LIST_LENGTH_KEY);
 	InvalidateList(LIST_SCALE_BAR);
     }
     void ToggleHitTestDebug() {
 	ToggleFlag(&m_HitTestDebug);
     }
-    void ToggleDegrees() { ToggleFlag(&m_Degrees); }
+    void ToggleRenderStats() {
+	ToggleFlag(&m_RenderStats);
+    }
+    void ToggleDegrees() {
+	ToggleFlag(&m_Degrees);
+	InvalidateList(LIST_GRADIENT_KEY);
+    }
     void TogglePercent() { ToggleFlag(&m_Percent); }
     void ToggleTubes() { ToggleFlag(&m_Tubes); }
     void TogglePerspective() { GLACanvas::TogglePerspective(); ForceRefresh(); }
     void ToggleSmoothShading();
     bool DisplayingBoundingBox() const { return m_BoundingBox; }
     void ToggleBoundingBox() { ToggleFlag(&m_BoundingBox); }
+    bool DisplayingTerrain() const { return m_Terrain; }
+    void ToggleTerrain();
     void ToggleFatFinger();
 
     bool GetMetric() const { return m_Metric; }
@@ -481,6 +536,8 @@ public:
     void AddPolylineDepth(const traverse & centreline);
     void AddPolylineDate(const traverse & centreline);
     void AddPolylineError(const traverse & centreline);
+    void AddPolylineGradient(const traverse & centreline);
+    void AddPolylineLength(const traverse & centreline);
     void AddQuadrilateral(const Vector3 &a, const Vector3 &b,
 			  const Vector3 &c, const Vector3 &d);
     void AddPolylineShadow(const traverse & centreline);
@@ -490,6 +547,10 @@ public:
 			      const Vector3 &c, const Vector3 &d);
     void AddQuadrilateralError(const Vector3 &a, const Vector3 &b,
 			       const Vector3 &c, const Vector3 &d);
+    void AddQuadrilateralGradient(const Vector3 &a, const Vector3 &b,
+				  const Vector3 &c, const Vector3 &d);
+    void AddQuadrilateralLength(const Vector3 &a, const Vector3 &b,
+			        const Vector3 &c, const Vector3 &d);
     void MoveViewer(double forward, double up, double right);
 
     void (GfxCore::* AddQuad)(const Vector3 &a, const Vector3 &b,
@@ -532,6 +593,11 @@ public:
 
     void ZoomBoxGo();
 
+    void parse_hgt_filename(const wxString & lc_name);
+    size_t parse_hdr(wxInputStream & is, unsigned long & skipbytes);
+    bool read_bil(wxInputStream & is, size_t size, unsigned long skipbytes);
+    bool LoadDEM(const wxString & file);
+
 private:
     DECLARE_EVENT_TABLE()
 };
