@@ -4,7 +4,7 @@
 //  OpenGL implementation for the GLA abstraction layer.
 //
 //  Copyright (C) 2002-2003,2005 Mark R. Shinwell
-//  Copyright (C) 2003,2004,2005,2006,2007,2010,2011,2012,2013,2014 Olly Betts
+//  Copyright (C) 2003,2004,2005,2006,2007,2010,2011,2012,2013,2014,2015 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -152,12 +152,12 @@ log_gl_error(const wxChar * str, GLenum error_code)
 //             BeginLines()/EndLines() block etc.)
 #define CHECK_GL_ERROR(M, F) do { \
     if (!opengl_initialised) { \
-	wxLogError(wxT(__FILE__":"STRING(__LINE__)": OpenGL not initialised before (call "F" in method "M")")); \
+	wxLogError(wxT(__FILE__ ":" STRING(__LINE__) ": OpenGL not initialised before (call " F " in method " M ")")); \
     } \
     GLenum error_code_ = glGetError(); \
     if (error_code_ != GL_NO_ERROR) { \
-	log_gl_error(wxT(__FILE__":"STRING(__LINE__)": OpenGL error: %s " \
-			 "(call "F" in method "M")"), error_code_); \
+	log_gl_error(wxT(__FILE__ ":" STRING(__LINE__) ": OpenGL error: %s " \
+			 "(call " F " in method " M ")"), error_code_); \
     } \
 } while (0)
 
@@ -911,6 +911,18 @@ void GLACanvas::DrawList(unsigned int l)
     }
 }
 
+void GLACanvas::DrawListZPrepass(unsigned int l)
+{
+    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
+    DrawList(l);
+    glDepthMask(GL_FALSE);
+    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+    glDepthFunc(GL_EQUAL);
+    DrawList(l);
+    glDepthMask(GL_TRUE);
+    glDepthFunc(GL_LESS);
+}
+
 void GLACanvas::DrawList2D(unsigned int l, glaCoord x, glaCoord y, Double rotation)
 {
     glMatrixMode(GL_PROJECTION);
@@ -981,7 +993,7 @@ void GLACanvas::DrawIndicatorText(int x, int y, const wxString& str)
     m_Font.write_string(str.data(), str.size());
 }
 
-void GLACanvas::GetTextExtent(const wxString& str, int * x_ext, int * y_ext)
+void GLACanvas::GetTextExtent(const wxString& str, int * x_ext, int * y_ext) const
 {
     m_Font.get_text_extent(str.data(), str.size(), x_ext, y_ext);
 }
@@ -1246,6 +1258,7 @@ void GLACanvas::EndCrosses()
 void GLACanvas::DrawCross(glaCoord x, glaCoord y, glaCoord z)
 {
     if (cross_method == SPRITE) {
+	// Draw a marker.
 	PlaceVertex(x, y, z);
     } else {
 	double X, Y, Z;
@@ -1255,16 +1268,16 @@ void GLACanvas::DrawCross(glaCoord x, glaCoord y, glaCoord z)
 	}
 	// Stuff behind us (in perspective view) will get clipped,
 	// but we can save effort with a cheap check here.
-	if (Z > 0) {
-	    // Round to integers before adding on the offsets for the
-	    // cross arms to avoid uneven crosses.
-	    X = rint(X);
-	    Y = rint(Y);
-	    PlaceVertex(X - 3, Y - 3, Z);
-	    PlaceVertex(X + 3, Y + 3, Z);
-	    PlaceVertex(X - 3, Y + 3, Z);
-	    PlaceVertex(X + 3, Y - 3, Z);
-	}
+	if (Z <= 0) return;
+
+	// Round to integers before adding on the offsets for the
+	// cross arms to avoid uneven crosses.
+	X = rint(X);
+	Y = rint(Y);
+	PlaceVertex(X - 3, Y - 3, Z);
+	PlaceVertex(X + 3, Y + 3, Z);
+	PlaceVertex(X - 3, Y + 3, Z);
+	PlaceVertex(X + 3, Y - 3, Z);
     }
 #ifdef GLA_DEBUG
     m_Vertices++;
@@ -1472,7 +1485,7 @@ void GLACanvas::ToggleTextured()
 	wxImage img;
 	wxString texture(wmsg_cfgpth());
 	texture += wxCONFIG_PATH_SEPARATOR;
-	texture += wxT("icons");
+	texture += wxT("images");
 	texture += wxCONFIG_PATH_SEPARATOR;
 	texture += wxT("texture.png");
 	if (!img.LoadFile(texture, wxBITMAP_TYPE_PNG)) {
