@@ -4,7 +4,7 @@
 //  Handlers for events relating to the display of a survey.
 //
 //  Copyright (C) 2000-2002,2005 Mark R. Shinwell
-//  Copyright (C) 2001,2003,2004,2005,2006,2011,2012,2014 Olly Betts
+//  Copyright (C) 2001,2003,2004,2005,2006,2011,2012,2014,2015 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -368,11 +368,6 @@ void GUIControl::OnLButtonUp(wxMouseEvent& event)
 	if (dragging != LEFT_DRAG)
 	    return;
 
-	if (event.GetPosition() == m_DragRealStart) {
-	    // Just a "click"...
-	    m_View->CheckHitTestGrid(m_DragStart, true);
-	}
-
 	if (event.MiddleIsDown()) {
 	    if (m_LastDrag == drag_ZOOM)
 		m_View->UnsetZoomBox();
@@ -399,6 +394,8 @@ void GUIControl::OnLButtonUp(wxMouseEvent& event)
 	m_View->DragFinished();
 
 	if (event.GetPosition() == m_DragRealStart) {
+	    // Just a "click"...
+	    m_View->CheckHitTestGrid(m_DragStart, true);
 	    RestoreCursor();
 	} else {
 	    HandleNonDrag(event.GetPosition());
@@ -544,6 +541,29 @@ void GUIControl::OnColourByError()
     }
 }
 
+void GUIControl::OnColourByGradient()
+{
+    if (m_View->ColouringBy() == COLOUR_BY_GRADIENT) {
+	m_View->SetColourBy(COLOUR_BY_NONE);
+    } else {
+	m_View->SetColourBy(COLOUR_BY_GRADIENT);
+    }
+}
+
+void GUIControl::OnColourByLength()
+{
+    if (m_View->ColouringBy() == COLOUR_BY_LENGTH) {
+	m_View->SetColourBy(COLOUR_BY_NONE);
+    } else {
+	m_View->SetColourBy(COLOUR_BY_LENGTH);
+    }
+}
+
+void GUIControl::OnColourByUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData());
+}
+
 void GUIControl::OnColourByDepthUpdate(wxUpdateUIEvent& cmd)
 {
     cmd.Enable(m_View->HasData());
@@ -562,6 +582,18 @@ void GUIControl::OnColourByErrorUpdate(wxUpdateUIEvent& cmd)
     cmd.Check(m_View->ColouringBy() == COLOUR_BY_ERROR);
 }
 
+void GUIControl::OnColourByGradientUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData());
+    cmd.Check(m_View->ColouringBy() == COLOUR_BY_GRADIENT);
+}
+
+void GUIControl::OnColourByLengthUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData());
+    cmd.Check(m_View->ColouringBy() == COLOUR_BY_LENGTH);
+}
+
 void GUIControl::OnShowCrosses()
 {
     m_View->ToggleCrosses();
@@ -709,26 +741,6 @@ void GUIControl::OnReverseDirectionOfRotationUpdate(wxUpdateUIEvent& cmd)
     cmd.Enable(m_View->HasData() && !m_View->IsExtendedElevation());
 }
 
-void GUIControl::OnSlowDown(bool accel)
-{
-    m_View->RotateSlower(accel);
-}
-
-void GUIControl::OnSlowDownUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && !m_View->IsExtendedElevation());
-}
-
-void GUIControl::OnSpeedUp(bool accel)
-{
-    m_View->RotateFaster(accel);
-}
-
-void GUIControl::OnSpeedUpUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && !m_View->IsExtendedElevation());
-}
-
 void GUIControl::OnStepOnceAnticlockwise(bool accel)
 {
     if (m_View->GetPerspective()) {
@@ -739,11 +751,6 @@ void GUIControl::OnStepOnceAnticlockwise(bool accel)
     m_View->ForceRefresh();
 }
 
-void GUIControl::OnStepOnceAnticlockwiseUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && !m_View->IsExtendedElevation() && !m_View->IsRotating());
-}
-
 void GUIControl::OnStepOnceClockwise(bool accel)
 {
     if (m_View->GetPerspective()) {
@@ -754,11 +761,6 @@ void GUIControl::OnStepOnceClockwise(bool accel)
     m_View->ForceRefresh();
 }
 
-void GUIControl::OnStepOnceClockwiseUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && !m_View->IsExtendedElevation() && !m_View->IsRotating());
-}
-
 void GUIControl::OnDefaults()
 {
     m_View->Defaults();
@@ -792,11 +794,6 @@ void GUIControl::OnHigherViewpoint(bool accel)
     m_View->ForceRefresh();
 }
 
-void GUIControl::OnHigherViewpointUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && m_View->CanRaiseViewpoint() && !m_View->IsExtendedElevation());
-}
-
 void GUIControl::OnLowerViewpoint(bool accel)
 {
     // Lower the viewpoint.
@@ -808,11 +805,6 @@ void GUIControl::OnLowerViewpoint(bool accel)
     m_View->ForceRefresh();
 }
 
-void GUIControl::OnLowerViewpointUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData() && m_View->CanLowerViewpoint() && !m_View->IsExtendedElevation());
-}
-
 void GUIControl::OnPlan()
 {
     // Switch to plan view.
@@ -832,11 +824,6 @@ void GUIControl::OnShiftDisplayDown(bool accel)
 	m_View->TranslateCave(0, accel ? 5 * DISPLAY_SHIFT : DISPLAY_SHIFT);
 }
 
-void GUIControl::OnShiftDisplayDownUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnShiftDisplayLeft(bool accel)
 {
     if (m_View->GetPerspective())
@@ -845,11 +832,6 @@ void GUIControl::OnShiftDisplayLeft(bool accel)
 	m_View->TranslateCave(accel ? -5 * DISPLAY_SHIFT : -DISPLAY_SHIFT, 0);
 }
 
-void GUIControl::OnShiftDisplayLeftUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnShiftDisplayRight(bool accel)
 {
     if (m_View->GetPerspective())
@@ -858,11 +840,6 @@ void GUIControl::OnShiftDisplayRight(bool accel)
 	m_View->TranslateCave(accel ? 5 * DISPLAY_SHIFT : DISPLAY_SHIFT, 0);
 }
 
-void GUIControl::OnShiftDisplayRightUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnShiftDisplayUp(bool accel)
 {
     if (m_View->GetPerspective())
@@ -871,11 +848,6 @@ void GUIControl::OnShiftDisplayUp(bool accel)
 	m_View->TranslateCave(0, accel ? -5 * DISPLAY_SHIFT : -DISPLAY_SHIFT);
 }
 
-void GUIControl::OnShiftDisplayUpUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnZoomIn(bool accel)
 {
     // Increase the scale.
@@ -888,11 +860,6 @@ void GUIControl::OnZoomIn(bool accel)
     }
 }
 
-void GUIControl::OnZoomInUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnZoomOut(bool accel)
 {
     // Decrease the scale.
@@ -905,11 +872,6 @@ void GUIControl::OnZoomOut(bool accel)
     }
 }
 
-void GUIControl::OnZoomOutUpdate(wxUpdateUIEvent& cmd)
-{
-    cmd.Enable(m_View->HasData());
-}
-
 void GUIControl::OnToggleScalebar()
 {
     m_View->ToggleScaleBar();
@@ -1189,11 +1151,11 @@ void GUIControl::OnKeyPress(wxKeyEvent &e)
 	    break;
 	case 'Z': case 'z':
 	    if (!m_View->IsExtendedElevation())
-		OnSpeedUp(e.GetModifiers() == wxMOD_SHIFT);
+		m_View->RotateFaster(e.GetModifiers() == wxMOD_SHIFT);
 	    break;
 	case 'X': case 'x':
 	    if (!m_View->IsExtendedElevation())
-		OnSlowDown(e.GetModifiers() == wxMOD_SHIFT);
+		m_View->RotateSlower(e.GetModifiers() == wxMOD_SHIFT);
 	    break;
 	case 'R': case 'r':
 	    if (!m_View->IsExtendedElevation())
@@ -1257,6 +1219,10 @@ void GUIControl::OnKeyPress(wxKeyEvent &e)
 	case WXK_ESCAPE:
 	    if (m_View->ShowingMeasuringLine()) {
 		OnCancelDistLine();
+	    } else if (m_View->IsFullScreen()) {
+		// Cancel full-screen mode on "Escape" if it isn't cancelling
+		// the measuring line.
+		m_View->FullScreenMode();
 	    }
 	    break;
 	case WXK_F2:
@@ -1265,21 +1231,30 @@ void GUIControl::OnKeyPress(wxKeyEvent &e)
 	case WXK_F3:
 	    m_View->ToggleHitTestDebug();
 	    break;
-	case WXK_F4:
+	case WXK_F4: {
+	    const wxChar * msg;
 #if wxDEBUG_LEVEL
 	    if (wxTheAssertHandler)
 		wxTheAssertHandler = NULL;
 	    else
 		wxSetDefaultAssertHandler();
-	    wxMessageBox(wxTheAssertHandler ? "Assertions enabled" : "Assertions disabled", "Aven Debug", wxOK | wxICON_INFORMATION);
+	    if (wxTheAssertHandler)
+		msg = wxT("Assertions enabled");
+	    else
+		msg = wxT("Assertions disabled");
 #else
-	    wxMessageBox("wxWidgets was built without assertions", "Aven Debug", wxOK | wxICON_INFORMATION);
+	    msg = wxT("wxWidgets was built without assertions");
 #endif
+	    wxMessageBox(msg, wxT("Aven Debug"), wxOK | wxICON_INFORMATION);
 	    break;
+	}
 	case WXK_F5:
 	    m_View->InvalidateAllLists();
 	    m_View->ForceRefresh();
 	    break;
+	case WXK_F6:
+	    m_View->ToggleRenderStats();
+	    break;
 	default:
 	    e.Skip();
     }
@@ -1307,3 +1282,14 @@ void GUIControl::OnViewBoundingBox()
 {
     m_View->ToggleBoundingBox();
 }
+
+void GUIControl::OnViewTerrainUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasTerrain());
+    cmd.Check(m_View->DisplayingTerrain());
+}
+
+void GUIControl::OnViewTerrain()
+{
+    m_View->ToggleTerrain();
+}
