@@ -2,7 +2,7 @@
  * Export from Aven as KML.
  */
 /* Copyright (C) 2012 Olaf Kähler
- * Copyright (C) 2012,2013,2014 Olly Betts
+ * Copyright (C) 2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -94,7 +94,8 @@ KML::passes() const
 }
 
 /* Initialise KML routines. */
-void KML::header(const char * title, const char *, time_t)
+void KML::header(const char * title, const char *, time_t,
+		 double, double, double, double, double, double)
 {
     fputs(
 "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
