@@ -2,7 +2,7 @@
  * Export from Aven as KML.
  */
 
-/* Copyright (C) 2005,2013,2014 Olly Betts
+/* Copyright (C) 2005,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,12 @@ class KML : public ExportFilter {
     projPJ pj_input, pj_output;
     bool in_trkseg;
   public:
-    KML(const char * input_datum);
+    explicit KML(const char * input_datum);
     ~KML();
     const int * passes() const;
-    void header(const char *, const char *, time_t);
+    void header(const char *, const char *, time_t,
+		double, double, double,
+		double, double, double);
     void line(const img_point *, const img_point *, bool, bool);
     void label(const img_point *, const char *, bool, int);
     void footer();
