@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2003,2005 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013,2014 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -49,12 +49,20 @@ using namespace std;
 #include <math.h>
 #include <time.h>
 
+#ifdef __WXMAC__
+// Currently on OS X, we force use of a generic toolbar, which needs to be put
+// in a sizer.
+# define USING_GENERIC_TOOLBAR
+#endif
+
 #define MARK_FIRST 1
 #define MARK_NEXT 2
 #define MARK_PREV 3
 
 enum {
-    menu_FILE_PAGE_SETUP = 1000,
+    menu_FILE_LOG = 1000,
+    menu_FILE_OPEN_TERRAIN,
+    menu_FILE_PAGE_SETUP,
     menu_FILE_SCREENSHOT,
     menu_FILE_EXPORT,
     menu_PRES_NEW,
@@ -73,23 +81,13 @@ enum {
     menu_ROTATION_START,
     menu_ROTATION_STOP,
     menu_ROTATION_TOGGLE,
-    menu_ROTATION_SPEED_UP,
-    menu_ROTATION_SLOW_DOWN,
     menu_ROTATION_REVERSE,
-    menu_ROTATION_STEP_CCW,
-    menu_ROTATION_STEP_CW,
     menu_ORIENT_MOVE_NORTH,
     menu_ORIENT_MOVE_EAST,
     menu_ORIENT_MOVE_SOUTH,
     menu_ORIENT_MOVE_WEST,
-    menu_ORIENT_SHIFT_LEFT,
-    menu_ORIENT_SHIFT_RIGHT,
-    menu_ORIENT_SHIFT_UP,
-    menu_ORIENT_SHIFT_DOWN,
     menu_ORIENT_PLAN,
     menu_ORIENT_ELEVATION,
-    menu_ORIENT_HIGHER_VP,
-    menu_ORIENT_LOWER_VP,
     menu_ORIENT_DEFAULTS,
     menu_VIEW_SHOW_LEGS,
     menu_VIEW_SPLAYS,
@@ -106,6 +104,7 @@ enum {
     menu_VIEW_STATUS_BAR,
     menu_VIEW_GRID,
     menu_VIEW_BOUNDING_BOX,
+    menu_VIEW_TERRAIN,
     menu_VIEW_SHOW_TUBES,
     menu_VIEW_PERSPECTIVE,
     menu_VIEW_SMOOTH_SHADING,
@@ -113,9 +112,12 @@ enum {
     menu_VIEW_FOG,
     menu_VIEW_SMOOTH_LINES,
     menu_VIEW_FULLSCREEN,
-    menu_VIEW_COLOUR_BY_DEPTH,
-    menu_VIEW_COLOUR_BY_DATE,
-    menu_VIEW_COLOUR_BY_ERROR,
+    menu_VIEW_COLOUR_BY,
+    menu_COLOUR_BY_DEPTH,
+    menu_COLOUR_BY_DATE,
+    menu_COLOUR_BY_ERROR,
+    menu_COLOUR_BY_GRADIENT,
+    menu_COLOUR_BY_LENGTH,
     menu_IND_COMPASS,
     menu_IND_CLINO,
     menu_IND_COLOUR_KEY,
@@ -137,7 +139,7 @@ class PointInfo : public Point {
 
 public:
     PointInfo() : Point(), date(-1) { }
-    PointInfo(const img_point & pt) : Point(pt), date(-1) { }
+    explicit PointInfo(const img_point & pt) : Point(pt), date(-1) { }
     PointInfo(const img_point & pt, int date_) : Point(pt), date(date_) { }
     PointInfo(const Point & p, int date_) : Point(p), date(date_) { }
     int GetDate() const { return date; }
@@ -188,6 +190,7 @@ class MainFrm : public wxFrame {
     int m_DateMin, m_DateExt;
     bool complete_dateinfo;
     GfxCore* m_Gfx;
+    wxWindow* m_Log;
     GUIControl* m_Control;
     int m_NumEntrances;
     int m_NumFixedPts;
@@ -204,13 +207,13 @@ class MainFrm : public wxFrame {
     wxNotebook* m_Notebook;
     AvenPresList* m_PresList;
     wxString m_File;
+public: // FIXME for m_cs_proj
     wxString m_Title, m_cs_proj, m_DateStamp;
+private:
     time_t m_DateStamp_numeric;
     wxChar separator; // character separating survey levels (often '.')
     Vector3 m_Offsets;
 
-    wxString icon_path;
-
     // Strings for status bar reporting of distances.
     wxString here_text, coords_text, dist_text, distfree_text;
 
@@ -223,9 +226,8 @@ class MainFrm : public wxFrame {
     PrefsDlg* m_PrefsDlg;
 #endif
 
-    void FillTree();
+    void FillTree(const wxString & root_name);
     bool ProcessSVXFile(const wxString & file);
-    bool LoadData(const wxString& file, const wxString& prefix);
 //    void FixLRUD(traverse & centreline);
     void CentreDataset(const Vector3 & vmin);
 
@@ -235,12 +237,23 @@ class MainFrm : public wxFrame {
 
     void UpdateStatusBar();
 
-    void AddToFileHistory(const wxString & file);
+#ifdef USING_GENERIC_TOOLBAR
+    wxToolBar * GetToolBar() const {
+	wxSizer * sizer = GetSizer();
+	if (!sizer) return NULL;
+	return (wxToolBar*)sizer->GetItem(size_t(0))->GetWindow();
+    }
+#endif
 
 public:
     MainFrm(const wxString& title, const wxPoint& pos, const wxSize& size);
 
+    // public for CavernLog.
+    bool LoadData(const wxString& file, const wxString& prefix);
+    void AddToFileHistory(const wxString & file);
+
     void InitialiseAfterLoad(const wxString & file);
+    void OnShowLog(wxCommandEvent& event);
 
     void OnMRUFile(wxCommandEvent& event);
     void OpenFile(const wxString& file, const wxString& survey = wxString());
@@ -259,7 +272,7 @@ public:
     void OnPresPauseUpdate(wxUpdateUIEvent& event);
     void OnPresStopUpdate(wxUpdateUIEvent& event);
     void OnPresExportMovieUpdate(wxUpdateUIEvent& event);
-    //void OnFileOpenTerrainUpdate(wxUpdateUIEvent& event);
+    void OnOpenTerrainUpdate(wxUpdateUIEvent& event);
 
     void DoFind();
     void OnFind(wxCommandEvent& event);
@@ -269,10 +282,11 @@ public:
     void OnHideUpdate(wxUpdateUIEvent& ui);
 
     void OnOpen(wxCommandEvent& event);
+    void OnOpenTerrain(wxCommandEvent&);
+    void HideLog(wxWindow * log_window);
     void OnScreenshot(wxCommandEvent& event);
     void OnScreenshotUpdate(wxUpdateUIEvent& event);
     void OnFilePreferences(wxCommandEvent& event);
-    void OnFileOpenTerrain(wxCommandEvent& event);
     void OnPrint(wxCommandEvent& event);
     void PrintAndExit();
     void OnPageSetup(wxCommandEvent& event);
@@ -305,6 +319,7 @@ public:
 	}
     }
 
+    void OnShowLogUpdate(wxUpdateUIEvent &ui) { ui.Enable(m_Log != NULL); }
     void OnPrintUpdate(wxUpdateUIEvent &ui) { ui.Enable(!m_File.empty()); }
     void OnExportUpdate(wxUpdateUIEvent &ui) { ui.Enable(!m_File.empty()); }
 
@@ -313,9 +328,12 @@ public:
     void OnPlanUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnPlanUpdate(event); }
     void OnElevationUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnElevationUpdate(event); }
     void OnDisplayOverlappingNamesUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnDisplayOverlappingNamesUpdate(event); }
+    void OnColourByUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByUpdate(event); }
     void OnColourByDepthUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByDepthUpdate(event); }
     void OnColourByDateUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByDateUpdate(event); }
     void OnColourByErrorUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByErrorUpdate(event); }
+    void OnColourByGradientUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByGradientUpdate(event); }
+    void OnColourByLengthUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnColourByLengthUpdate(event); }
     void OnShowCrossesUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowCrossesUpdate(event); }
     void OnShowEntrancesUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowEntrancesUpdate(event); }
     void OnShowFixedPtsUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowFixedPtsUpdate(event); }
@@ -333,23 +351,12 @@ public:
     void OnMoveWestUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnMoveWestUpdate(event); }
     void OnToggleRotationUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnToggleRotationUpdate(event); }
     void OnReverseControlsUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnReverseControlsUpdate(event); }
-    void OnSlowDownUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnSlowDownUpdate(event); }
-    void OnSpeedUpUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnSpeedUpUpdate(event); }
-    void OnStepOnceAnticlockwiseUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnStepOnceAnticlockwiseUpdate(event); }
-    void OnStepOnceClockwiseUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnStepOnceClockwiseUpdate(event); }
-    void OnHigherViewpointUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnHigherViewpointUpdate(event); }
-    void OnLowerViewpointUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnLowerViewpointUpdate(event); }
-    void OnShiftDisplayDownUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShiftDisplayDownUpdate(event); }
-    void OnShiftDisplayLeftUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShiftDisplayLeftUpdate(event); }
-    void OnShiftDisplayRightUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShiftDisplayRightUpdate(event); }
-    void OnShiftDisplayUpUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShiftDisplayUpUpdate(event); }
-    void OnZoomInUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnZoomInUpdate(event); }
-    void OnZoomOutUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnZoomOutUpdate(event); }
     void OnToggleScalebarUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnToggleScalebarUpdate(event); }
     void OnToggleColourKeyUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnToggleColourKeyUpdate(event); }
     void OnViewCompassUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewCompassUpdate(event); }
     void OnViewGridUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewGridUpdate(event); }
     void OnViewBoundingBoxUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewBoundingBoxUpdate(event); }
+    void OnViewTerrainUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewTerrainUpdate(event); }
     void OnViewClinoUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewClinoUpdate(event); }
     void OnViewPerspectiveUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewPerspectiveUpdate(event); }
     void OnViewSmoothShadingUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnViewSmoothShadingUpdate(event); }
@@ -368,6 +375,8 @@ public:
     void OnColourByDepth(wxCommandEvent&) { if (m_Control) m_Control->OnColourByDepth(); }
     void OnColourByDate(wxCommandEvent&) { if (m_Control) m_Control->OnColourByDate(); }
     void OnColourByError(wxCommandEvent&) { if (m_Control) m_Control->OnColourByError(); }
+    void OnColourByGradient(wxCommandEvent&) { if (m_Control) m_Control->OnColourByGradient(); }
+    void OnColourByLength(wxCommandEvent&) { if (m_Control) m_Control->OnColourByLength(); }
     void OnShowCrosses(wxCommandEvent&) { if (m_Control) m_Control->OnShowCrosses(); }
     void OnShowEntrances(wxCommandEvent&) { if (m_Control) m_Control->OnShowEntrances(); }
     void OnShowFixedPts(wxCommandEvent&) { if (m_Control) m_Control->OnShowFixedPts(); }
@@ -384,24 +393,13 @@ public:
     void OnMoveWest(wxCommandEvent&) { if (m_Control) m_Control->OnMoveWest(); }
     void OnToggleRotation(wxCommandEvent&) { if (m_Control) m_Control->OnToggleRotation(); }
     void OnReverseControls(wxCommandEvent&) { if (m_Control) m_Control->OnReverseControls(); }
-    void OnSlowDown(wxCommandEvent&) { if (m_Control) m_Control->OnSlowDown(); }
-    void OnSpeedUp(wxCommandEvent&) { if (m_Control) m_Control->OnSpeedUp(); }
-    void OnStepOnceAnticlockwise(wxCommandEvent&) { if (m_Control) m_Control->OnStepOnceAnticlockwise(); }
-    void OnStepOnceClockwise(wxCommandEvent&) { if (m_Control) m_Control->OnStepOnceClockwise(); }
-    void OnHigherViewpoint(wxCommandEvent&) { if (m_Control) m_Control->OnHigherViewpoint(); }
-    void OnLowerViewpoint(wxCommandEvent&) { if (m_Control) m_Control->OnLowerViewpoint(); }
-    void OnShiftDisplayDown(wxCommandEvent&) { if (m_Control) m_Control->OnShiftDisplayDown(); }
-    void OnShiftDisplayLeft(wxCommandEvent&) { if (m_Control) m_Control->OnShiftDisplayLeft(); }
-    void OnShiftDisplayRight(wxCommandEvent&) { if (m_Control) m_Control->OnShiftDisplayRight(); }
-    void OnShiftDisplayUp(wxCommandEvent&) { if (m_Control) m_Control->OnShiftDisplayUp(); }
-    void OnZoomIn(wxCommandEvent&) { if (m_Control) m_Control->OnZoomIn(); }
-    void OnZoomOut(wxCommandEvent&) { if (m_Control) m_Control->OnZoomOut(); }
     void OnToggleScalebar(wxCommandEvent&) { if (m_Control) m_Control->OnToggleScalebar(); }
     void OnToggleColourKey(wxCommandEvent&) { if (m_Control) m_Control->OnToggleColourKey(); }
     void OnViewCompass(wxCommandEvent&) { if (m_Control) m_Control->OnViewCompass(); }
     void OnViewClino(wxCommandEvent&) { if (m_Control) m_Control->OnViewClino(); }
     void OnViewGrid(wxCommandEvent&) { if (m_Control) m_Control->OnViewGrid(); }
     void OnViewBoundingBox(wxCommandEvent&) { if (m_Control) m_Control->OnViewBoundingBox(); }
+    void OnViewTerrain(wxCommandEvent&) { if (m_Control) m_Control->OnViewTerrain(); }
     void OnViewPerspective(wxCommandEvent&) { if (m_Control) m_Control->OnViewPerspective(); }
     void OnViewSmoothShading(wxCommandEvent&) { if (m_Control) m_Control->OnViewSmoothShading(); }
     void OnViewTextured(wxCommandEvent&) { if (m_Control) m_Control->OnViewTextured(); }
@@ -473,6 +471,8 @@ public:
 
     const Vector3 & GetOffset() const { return m_Offsets; }
 
+    wxChar GetSeparator() const { return separator; }
+
     list<traverse>::const_iterator traverses_begin() const {
 	return traverses.begin();
     }
@@ -548,7 +548,7 @@ class AvenAllowOnTop {
 #if defined __WXGTK__ && !wxCHECK_VERSION(2,8,10)
 	MainFrm * mainfrm;
     public:
-	AvenAllowOnTop(MainFrm * mainfrm_) {
+	explicit AvenAllowOnTop(MainFrm * mainfrm_) {
 	    if (mainfrm_ && mainfrm_->IsFullScreen()) {
 		mainfrm = mainfrm_;
 		mainfrm->ViewFullScreen();
