@@ -1,6 +1,6 @@
 /* message.c
  * Fairly general purpose message and error routines
- * Copyright (C) 1993-2003,2004,2005,2006,2007,2010,2011,2012,2014 Olly Betts
+ * Copyright (C) 1993-2003,2004,2005,2006,2007,2010,2011,2012,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,16 +43,6 @@
 # include "aven.h"
 #endif
 
-#ifdef HAVE_SIGNAL
-# ifdef HAVE_SETJMP_H
-#  include <setjmp.h>
-static jmp_buf jmpbufSignal;
-#  include <signal.h>
-# else
-#  undef HAVE_SIGNAL
-# endif
-#endif
-
 #if OS_WIN32
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
@@ -189,58 +179,6 @@ osfree(void *p)
 }
 #endif
 
-#ifdef HAVE_SIGNAL
-
-static int sigReceived;
-
-/* for systems not using autoconf, assume the signal handler returns void
- * unless specified elsewhere */
-#ifndef RETSIGTYPE
-# define RETSIGTYPE void
-#endif
-
-static CDECL RETSIGTYPE
-report_sig(int sig)
-{
-   sigReceived = sig;
-   longjmp(jmpbufSignal, 1);
-}
-
-static void
-init_signals(void)
-{
-   int en;
-   if (!setjmp(jmpbufSignal)) {
-      signal(SIGABRT, report_sig); /* abnormal termination eg abort() */
-      signal(SIGFPE,  report_sig); /* arithmetic error eg /0 or overflow */
-      signal(SIGILL,  report_sig); /* illegal function image eg illegal instruction */
-      signal(SIGSEGV, report_sig); /* illegal storage access eg access outside memory limits */
-      return;
-   }
-
-   /* Remove that signal handler to avoid the possibility of an infinite loop.
-    */
-   signal(sigReceived, SIG_DFL);
-
-   switch (sigReceived) {
-      /* TRANSLATORS: Program will exit shortly after printing this */
-      case SIGABRT: en = /*Abnormal termination*/90; break;
-      case SIGFPE:  en = /*Arithmetic error*/91; break;
-      /* TRANSLATORS: Something is badly wrong -- the CPU tried to execute bad
-       * opcodes -- corrupted program? */
-      case SIGILL:  en = /*Illegal instruction*/92; break;
-      case SIGSEGV: en = /*Bad memory access*/94; break;
-      default:      en = /*Unknown signal received*/97; break;
-   }
-   fputsnl(msg(en), STDERR);
-
-   /* Any of the signals we catch indicates a bug */
-   fatalerror(/*Bug in program detected! Please report this to the authors*/11);
-
-   exit(EXIT_FAILURE);
-}
-#endif
-
 static int
 default_charset(void)
 {
@@ -274,7 +212,7 @@ default_charset(void)
    }
 
    if (p) {
-      char *q = strchr(p, '.');
+      const char *q = strchr(p, '.');
       if (q) p = q + 1;
    }
 
@@ -467,6 +405,8 @@ add_unicode(int charset, unsigned char *p, int value)
       /* MS Windows rough equivalent to ISO-8859-2 */
       if (value >= 0x80) {
 	 int v = 0;
+	 /* This mapping is complete - there are 5 unused positions:
+	  * 0x81 0x83 0x88 0x90 0x98 */
 	 switch (value) {
 	    case 0xa0: case 0xa4: case 0xa6: case 0xa7: case 0xa8: case 0xa9:
 	    case 0xab: case 0xac: case 0xad: case 0xae: case 0xb0: case 0xb1:
@@ -560,16 +500,38 @@ add_unicode(int charset, unsigned char *p, int value)
       return 1;
    case CHARSET_WINCP1252:
       /* MS Windows extensions to ISO-8859-1 */
+      /* This mapping is complete - there are 5 unused positions:
+       * 0x81 0x8d 0x8f 0x90 0x9d */
       switch (value) {
        case 0x2026: value = 0x85; break; /* hellip */
-       case 0x160: value = 0x8a; break; /* Scaron */
-       case 0x152: value = 0x8c; break; /* OElig */
-       case 0x17d: value = 0x8e; break; /* Zcaron */
-       case 0x161: value = 0x9a; break; /* scaron */
-       case 0x153: value = 0x9c; break; /* oelig */
-       case 0x17e: value = 0x9e; break; /* zcaron */
+       case 0x0160: value = 0x8a; break; /* Scaron */
+       case 0x0152: value = 0x8c; break; /* OElig */
+       case 0x017d: value = 0x8e; break; /* Zcaron */
+       case 0x2019: value = 0x92; break; /* lsquo */
+       case 0x201c: value = 0x93; break; /* ldquo */
+       case 0x201d: value = 0x94; break; /* rdquo */
+       case 0x0161: value = 0x9a; break; /* scaron */
+       case 0x0153: value = 0x9c; break; /* oelig */
+       case 0x017e: value = 0x9e; break; /* zcaron */
 #if 0
-      /* there are a few other obscure ones we don't currently need */
+       /* there are a few other obscure ones we don't currently need */
+       case 0x20ac: value = 0x80; break;
+       case 0x201a: value = 0x82; break;
+       case 0x0192: value = 0x83; break;
+       case 0x201e: value = 0x84; break;
+       case 0x2020: value = 0x86; break;
+       case 0x2021: value = 0x87; break;
+       case 0x02c6: value = 0x88; break;
+       case 0x2030: value = 0x89; break;
+       case 0x2039: value = 0x8b; break;
+       case 0x2018: value = 0x91; break;
+       case 0x2022: value = 0x95; break;
+       case 0x2013: value = 0x96; break;
+       case 0x2014: value = 0x97; break;
+       case 0x02dc: value = 0x98; break;
+       case 0x2122: value = 0x99; break;
+       case 0x203a: value = 0x9b; break;
+       case 0x0178: value = 0x9f; break;
 #endif
       }
       if (value < 0x100) {
@@ -723,9 +685,15 @@ add_unicode(int charset, unsigned char *p, int value)
       *p = '.'; p[1] = '.'; p[2] = '.'; return 3;
     case 0x2192: /* &rarr; */
       *p = '-'; p[1] = '>'; return 2;
+    case 0x1d4d: /* gradient symbol */
+      *p = 'g'; p[1] = 'r'; p[2] = 'd'; return 3;
+    case 0x221e: /* infinity symbol */
+      *p = 'i'; p[1] = 'n'; p[2] = 'f'; return 3;
    }
 #ifdef DEBUG
-   fprintf(stderr, "failed to transliterate\n");
+   /* 169 is reported (copyright symbol), but there isn't a good <= 2 ASCII
+    * character transliteration for that, so we handle that elsewhere. */
+   fprintf(stderr, "failed to transliterate codepoint %d\n", value);
 #endif
    return 0;
 }
@@ -1101,9 +1069,6 @@ macosx_got_msg:
 		case SUBLANG_GERMAN_SWISS:
 		  msg_lang = "de_CH";
 		  break;
-		case SUBLANG_GERMAN:
-		  msg_lang = "de_DE";
-		  break;
 		default:
 		  msg_lang = "de";
 	       }
@@ -1111,6 +1076,9 @@ macosx_got_msg:
 	     case LANG_GREEK:
 	       msg_lang = "el";
 	       break;
+	     case LANG_HUNGARIAN:
+	       msg_lang = "hu";
+	       break;
 	     case LANG_INDONESIAN:
 	       msg_lang = "id";
 	       break;
@@ -1179,27 +1147,18 @@ macosx_got_msg:
 #endif
 
    select_charset(default_charset());
-
-#ifdef HAVE_SIGNAL
-   /* Initialise signal handlers only after the messages have been as we need
-    * the messages to usefully handle the signals. */
-   init_signals();
-#endif
 }
 
-/* Message may be overwritten by next call
- * (but not in current implementation) */
 const char *
 msg(int en)
 {
    /* NB can't use SVX_ASSERT here! */
-   static char badbuf[256];
    if (dontextract && en >= 1000 && en < 1000 + N_DONTEXTRACTMSGS)
       return dontextract[en - 1000];
    if (!msg_array) {
       if (en != 1)  {
-	 sprintf(badbuf, "Message %d requested before fully initialised\n", en);
-	 return badbuf;
+	 fprintf(STDERR, "Message %d requested before fully initialised\n", en);
+	 return "Message requested before fully initialised\n";
       }
       /* this should be the only other message which can be requested before
        * the message file is opened and read... */
@@ -1209,8 +1168,8 @@ msg(int en)
    }
 
    if (en < 0 || en >= num_msgs) {
-      sprintf(badbuf, "Message %d out of range\n", en);
-      return badbuf;
+      fprintf(STDERR, "Message %d out of range\n", en);
+      return "Message out of range\n";
    }
 
    if (en == 0) {
@@ -1222,13 +1181,6 @@ msg(int en)
    return msg_array[en];
 }
 
-/* returns persistent copy of message */
-const char *
-msgPerm(int en)
-{
-   return msg(en);
-}
-
 void
 v_report(int severity, const char *fnm, int line, int col, int en, va_list ap)
 {
@@ -1236,6 +1188,7 @@ v_report(int severity, const char *fnm, int line, int col, int en, va_list ap)
    (void)col;
    aven_v_report(severity, fnm, line, en, ap);
 #else
+   const char * level;
    if (fnm) {
       fputs(fnm, STDERR);
       if (line) fprintf(STDERR, ":%d", line);
@@ -1248,9 +1201,14 @@ v_report(int severity, const char *fnm, int line, int col, int en, va_list ap)
    if (severity == 0) {
       /* TRANSLATORS: Indicates a warning message e.g.:
        * "spoon.svx:12: warning: *prefix is deprecated" */
-      fputs(msg(/*warning*/4), STDERR);
-      fputs(": ", STDERR);
+      level = msg(/*warning*/4);
+   } else {
+      /* TRANSLATORS: Indicates an error message e.g.:
+       * "spoon.svx:13:4: error: Field may not be omitted" */
+      level = msg(/*error*/93);
    }
+   fputs(level, STDERR);
+   fputs(": ", STDERR);
 
    vfprintf(STDERR, msg(en), ap);
    fputnl(STDERR);
